@@ -6,7 +6,6 @@ from sqlalchemy import CheckConstraint
 from sqlalchemy import Column
 from sqlalchemy import DateTime
 from sqlalchemy import Enum
-from sqlalchemy import exc
 from sqlalchemy import ForeignKey
 from sqlalchemy import ForeignKeyConstraint
 from sqlalchemy import func
@@ -26,6 +25,7 @@ from sqlalchemy.schema import CreateTable
 from sqlalchemy.sql import column
 from sqlalchemy.sql import text
 
+from alembic import testing
 from alembic.ddl import sqlite
 from alembic.operations import Operations
 from alembic.operations.batch import ApplyBatchImpl
@@ -40,8 +40,16 @@ from alembic.testing import TestBase
 from alembic.testing.fixtures import op_fixture
 from alembic.util import exc as alembic_exc
 from alembic.util.sqla_compat import _select
+from alembic.util.sqla_compat import has_computed
+from alembic.util.sqla_compat import has_identity
 from alembic.util.sqla_compat import sqla_14
 
+if has_computed:
+    from alembic.util.sqla_compat import Computed
+
+if has_identity:
+    from alembic.util.sqla_compat import Identity
+
 
 class BatchApplyTest(TestBase):
     def setUp(self):
@@ -1335,6 +1343,17 @@ class BatchRoundTripTest(TestBase):
             t.create(self.conn)
             return t
 
+    def _ck_constraint_fixture(self):
+        with self.conn.begin():
+            t = Table(
+                "ck_table",
+                self.metadata,
+                Column("id", Integer, nullable=False),
+                CheckConstraint("id is not NULL", name="ck"),
+            )
+            t.create(self.conn)
+            return t
+
     def _datetime_server_default_fixture(self):
         return func.datetime("now", "localtime")
 
@@ -1418,7 +1437,9 @@ class BatchRoundTripTest(TestBase):
     def test_drop_col_schematype(self):
         self._boolean_fixture()
         with self.op.batch_alter_table("hasbool") as batch_op:
-            batch_op.drop_column("x")
+            batch_op.drop_column(
+                "x", existing_type=Boolean(create_constraint=True, name="ck1")
+            )
         insp = inspect(config.db)
 
         assert "x" not in (c["name"] for c in insp.get_columns("hasbool"))
@@ -1650,22 +1671,28 @@ class BatchRoundTripTest(TestBase):
             eq_(pk_const["name"], "newpk")
         eq_(pk_const["constrained_columns"], ["a", "b"])
 
-    @config.requirements.check_constraints_w_enforcement
+    @config.requirements.check_constraint_reflection
     def test_add_ck_constraint(self):
         with self.op.batch_alter_table("foo", recreate="always") as batch_op:
             batch_op.create_check_constraint("newck", text("x > 0"))
 
-        # we dont support reflection of CHECK constraints
-        # so test this by just running invalid data in
-        foo = self.metadata.tables["foo"]
-
-        assert_raises_message(
-            exc.IntegrityError,
-            "newck",
-            self.conn.execute,
-            foo.insert(),
-            {"id": 6, "data": 5, "x": -2},
+        ck_consts = inspect(self.conn).get_check_constraints("foo")
+        ck_consts[0]["sqltext"] = re.sub(
+            r"[\'\"`\(\)]", "", ck_consts[0]["sqltext"]
         )
+        eq_(ck_consts, [{"sqltext": "x > 0", "name": "newck"}])
+
+    @config.requirements.check_constraint_reflection
+    def test_drop_ck_constraint(self):
+        self._ck_constraint_fixture()
+
+        with self.op.batch_alter_table(
+            "ck_table", recreate="always"
+        ) as batch_op:
+            batch_op.drop_constraint("ck", "check")
+
+        ck_consts = inspect(self.conn).get_check_constraints("ck_table")
+        eq_(ck_consts, [])
 
     @config.requirements.unnamed_constraints
     def test_drop_foreign_key(self):
@@ -1703,6 +1730,36 @@ class BatchRoundTripTest(TestBase):
             ]
         )
 
+    def _assert_table_comment(self, tname, comment):
+        insp = inspect(config.db)
+
+        tcomment = insp.get_table_comment(tname)
+        eq_(tcomment, {"text": comment})
+
+    @config.requirements.comments
+    def test_add_table_comment(self):
+        with self.op.batch_alter_table("foo") as batch_op:
+            batch_op.create_table_comment("some comment")
+
+        self._assert_table_comment("foo", "some comment")
+
+        with self.op.batch_alter_table("foo") as batch_op:
+            batch_op.create_table_comment(
+                "some new comment", existing_comment="some comment"
+            )
+
+        self._assert_table_comment("foo", "some new comment")
+
+    @config.requirements.comments
+    def test_drop_table_comment(self):
+        with self.op.batch_alter_table("foo") as batch_op:
+            batch_op.create_table_comment("some comment")
+
+        with self.op.batch_alter_table("foo") as batch_op:
+            batch_op.drop_table_comment(existing_comment="some comment")
+
+        self._assert_table_comment("foo", None)
+
     def _assert_column_comment(self, tname, cname, comment):
         insp = inspect(config.db)
 
@@ -1815,6 +1872,31 @@ class BatchRoundTripTest(TestBase):
             [{"id": 1, "bflag": True}, {"id": 2, "bflag": False}], "bar"
         )
 
+    #    @config.requirements.check_constraint_reflection
+    def test_rename_column_boolean_named_ck(self):
+        bar = Table(
+            "bar",
+            self.metadata,
+            Column("id", Integer, primary_key=True),
+            Column("flag", Boolean(create_constraint=True, name="ck1")),
+            mysql_engine="InnoDB",
+        )
+        with self.conn.begin():
+            bar.create(self.conn)
+            self.conn.execute(bar.insert(), {"id": 1, "flag": True})
+            self.conn.execute(bar.insert(), {"id": 2, "flag": False})
+
+        with self.op.batch_alter_table("bar", recreate="always") as batch_op:
+            batch_op.alter_column(
+                "flag",
+                new_column_name="bflag",
+                existing_type=Boolean(create_constraint=True, name="ck1"),
+            )
+
+        self._assert_data(
+            [{"id": 1, "bflag": True}, {"id": 2, "bflag": False}], "bar"
+        )
+
     @config.requirements.non_native_boolean
     def test_rename_column_non_native_boolean_no_ck(self):
         bar = Table(
@@ -1901,6 +1983,78 @@ class BatchRoundTripTest(TestBase):
             ["id", "data", "x", "data2"],
         )
 
+    def test_add_column_auto_server_default_calculated(self):
+        """test #883"""
+        with self.op.batch_alter_table("foo") as batch_op:
+            batch_op.add_column(
+                Column(
+                    "data2",
+                    DateTime(),
+                    server_default=self._datetime_server_default_fixture(),
+                )
+            )
+
+        self._assert_data(
+            [
+                {"id": 1, "data": "d1", "x": 5, "data2": mock.ANY},
+                {"id": 2, "data": "22", "x": 6, "data2": mock.ANY},
+                {"id": 3, "data": "8.5", "x": 7, "data2": mock.ANY},
+                {"id": 4, "data": "9.46", "x": 8, "data2": mock.ANY},
+                {"id": 5, "data": "d5", "x": 9, "data2": mock.ANY},
+            ]
+        )
+        eq_(
+            [col["name"] for col in inspect(config.db).get_columns("foo")],
+            ["id", "data", "x", "data2"],
+        )
+
+    @testing.combinations((True,), (False,))
+    @testing.exclusions.only_on("sqlite")
+    @config.requirements.computed_columns
+    def test_add_column_auto_generated(self, persisted):
+        """test #883"""
+        with self.op.batch_alter_table("foo") as batch_op:
+            batch_op.add_column(
+                Column(
+                    "data2", Integer, Computed("1 + 1", persisted=persisted)
+                )
+            )
+
+        self._assert_data(
+            [
+                {"id": 1, "data": "d1", "x": 5, "data2": 2},
+                {"id": 2, "data": "22", "x": 6, "data2": 2},
+                {"id": 3, "data": "8.5", "x": 7, "data2": 2},
+                {"id": 4, "data": "9.46", "x": 8, "data2": 2},
+                {"id": 5, "data": "d5", "x": 9, "data2": 2},
+            ]
+        )
+        eq_(
+            [col["name"] for col in inspect(config.db).get_columns("foo")],
+            ["id", "data", "x", "data2"],
+        )
+
+    @config.requirements.identity_columns
+    def test_add_column_auto_identity(self):
+        """test #883"""
+
+        self._no_pk_fixture()
+
+        with self.op.batch_alter_table("nopk") as batch_op:
+            batch_op.add_column(Column("id", Integer, Identity()))
+
+        self._assert_data(
+            [
+                {"a": 1, "b": 2, "c": 3, "id": 1},
+                {"a": 2, "b": 4, "c": 5, "id": 2},
+            ],
+            tablename="nopk",
+        )
+        eq_(
+            [col["name"] for col in inspect(config.db).get_columns("foo")],
+            ["id", "data", "x"],
+        )
+
     def test_add_column_insert_before_recreate(self):
         with self.op.batch_alter_table("foo", recreate="always") as batch_op:
             batch_op.add_column(
